using System;
using System.IO;
using System.Threading;
using Microsoft.SPOT.Hardware;
using GHIElectronics.NETMF.Hardware;

namespace GHIElectronics.NETMF.FEZ
{
    public static partial class FEZ_Components
    {
        public class SerialELCD162 : IDisposable
        {   // Philippe Mariano le 24/06/2013
            // Adaptation de la classe dveloppe par GHI ELECTRONICS 
            //  un afficheur srie COMFILE ELCD-162 
            // les codes personnaliss ne sont pas grs
            // https://www.ghielectronics.com/community/codeshare/entry/26

            OutputCompare oc;
            const int MAX_TIMINGS_BUFFER_SIZE = 10;
            uint[] buffer = new uint[MAX_TIMINGS_BUFFER_SIZE];
            const int BAUD_RATE = 19200; // Pour Comfile ELCD-162
            const int BIT_TIME_US = 1 * 1000 * 1000 / BAUD_RATE;
            readonly int BYTE_TIME_MS;

            public SerialELCD162(byte pin)  // Pour Comfile ELCD-162
            {
                BYTE_TIME_MS = (int)Math.Ceiling((double)BIT_TIME_US * MAX_TIMINGS_BUFFER_SIZE / 1000);
                oc = new OutputCompare((Cpu.Pin)pin, true, MAX_TIMINGS_BUFFER_SIZE);
            }

            public void Dispose()
            {
                oc.Dispose();
                buffer = null;
            }

            private void SendByte(byte b)
            {
                bool currentPinState;
                int currentBufferIndex = 0;
                uint currentStateTiming;

                // start bit
                currentPinState = false;
                currentStateTiming = BIT_TIME_US;

                // data bits
                for (int i = 0; i < 8; i++)
                {
                    bool neededState = (b & (1 << i)) != 0;

                    if (neededState != currentPinState)
                    {
                        buffer[currentBufferIndex] = currentStateTiming;
                        currentStateTiming = BIT_TIME_US;
                        currentPinState = neededState;
                        currentBufferIndex++;
                    }
                    else
                    {
                        currentStateTiming += BIT_TIME_US;
                    }
                }

                // stop bit
                if (currentPinState != true)
                {
                    buffer[currentBufferIndex] = currentStateTiming;
                    currentBufferIndex++;
                }

                oc.Set(false, buffer, 0, currentBufferIndex, false);

                // wait till data is sent
                Thread.Sleep(BYTE_TIME_MS);
            }

            public void PutChar(char c)
            {
                SendByte((byte)c);
            }

            public void PutString(string str) // Pour Comfile ELCD-162
            {
                SendByte(0xA2);
                for (int i = 0; i < str.Length; i++)
                    PutChar(str[i]);
                SendByte(0);
            }

            public void ClearScreen() // Pour Comfile ELCD-162
            {
                SendByte(0xA3);
                SendByte(0x01);
            }

            public void CursorOn()  // Pour Comfile ELCD-162
            {
                SendByte(0xA3);
                SendByte(0x0E);
            }

            public void CursorOff() // Pour Comfile ELCD-162
            {
                SendByte(0xA3);
                SendByte(0x0C);
            }

            public void SetCursor(byte x_pos, byte y_pos) // Pour Comfile ELCD-162
            {
                SendByte(0xA1);
                SendByte(x_pos);
                SendByte(y_pos);
            }
        }
    }
}
